
/*
 * The contents of this file are subject to the terms
 * of the Common Development and Distribution License
 * (the License).  You may not use this file except in
 * compliance with the License.
 *
 * You can obtain a copy of the license at
 * https://glassfish.dev.java.net/public/CDDLv1.0.html or
 * glassfish/bootstrap/legal/CDDLv1.0.txt.
 * See the License for the specific language governing
 * permissions and limitations under the License.
 *
 * When distributing Covered Code, include this CDDL
 * Header Notice in each file and include the License file
 * at glassfish/bootstrap/legal/CDDLv1.0.txt.
 * If applicable, add the following below the CDDL Header,
 * with the fields enclosed by brackets [] replaced by
 * you own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Copyright 2006 Sun Microsystems, Inc. All rights reserved.
 *
 * Portions Copyright Apache Software Foundation.
 */

package com.sun.grizzly.grizzlet;

import com.sun.grizzly.container.GrizzletRequest;
import com.sun.grizzly.container.GrizzletResponse;
import java.io.IOException;
import java.util.logging.Logger;

/**
 * Simple Grizzlet that implement the logic to build a Chat application.
 *
 * @author Jeanfrancois Arcand
 * @autor TAKAI Naoto (Orginial author for the Comet based Chat).
 */
public class ChatGrizzlet implements Grizzlet {
    private static final String BEGIN_SCRIPT_TAG = "<script type='text/javascript'>\n";
    private static final String END_SCRIPT_TAG = "</script>\n";
    private static final Logger logger = Logger.getLogger("Grizzly");
    private static final long serialVersionUID = -2919167206889576860L; 
    
       
    public ChatGrizzlet() {
    }

 
    public void onRequest(AsyncConnection ac) throws IOException {
        GrizzletRequest req = ac.getRequest();
        GrizzletResponse res = ac.getResponse();
                
        res.setContentType("text/html");
        res.addHeader("Cache-Control", "private");
        res.addHeader("Pragma", "no-cache");                
        if (ac.isGet()){                             
            // for IE
            res.write("<!-- Comet is a programming technique that enables web " +
                    "servers to send data to the client without having any need " +
                    "for the client to request it. -->\n");
            res.flush();
            
            //
            // Mark this connection as suspended.
            //
            ac.suspend();
            return;
        } else if (ac.isPost()){
            res.setCharacterEncoding("UTF-8");
            String action = req.getParameterValues("action")[0];
            String name = req.getParameterValues("name")[0];

            if ("login".equals(action)) {
                ac.push(
                        BEGIN_SCRIPT_TAG + toJsonp("System Message", name 
                            + " has joined.") + END_SCRIPT_TAG);
                res.write("success");                 
                res.flush();
            } else if ("post".equals(action)) {                                
                String message = req.getParameterValues("message")[0];
                ac.push(BEGIN_SCRIPT_TAG 
                        + toJsonp(name, message) + END_SCRIPT_TAG);
                res.write("success");                 
                res.flush();                
            } else {
                res.setStatus(422);
                res.setMessage("Unprocessable Entity");
                                                
                res.write("success");                 
                res.flush();
            }
        }          
    }
    
    
    public void onPush(AsyncConnection ac) throws IOException{        
        GrizzletRequest req = ac.getRequest();
        GrizzletResponse res = ac.getResponse();
                
        if (ac.isResuming()){
            String script = BEGIN_SCRIPT_TAG 
                + "window.parent.app.listen();\n" 
                + END_SCRIPT_TAG;

            res.write(script);
            res.flush();
            res.finish();   
            return;            
        } else if (ac.hasPushEvent()){
            res.write(ac.getPushEvent().toString());
            res.flush();  
            return;
        }   
    }
    
    
    private String escape(String orig) {
        StringBuffer buffer = new StringBuffer(orig.length());

        for (int i = 0; i < orig.length(); i++) {
            char c = orig.charAt(i);
            switch (c) {
            case '\b':
                buffer.append("\\b");
                break;
            case '\f':
                buffer.append("\\f");
                break;
            case '\n':
                buffer.append("<br />");
                break;
            case '\r':
                // ignore
                break;
            case '\t':
                buffer.append("\\t");
                break;
            case '\'':
                buffer.append("\\'");
                break;
            case '\"':
                buffer.append("\\\"");
                break;
            case '\\':
                buffer.append("\\\\");
                break;
            case '<':
                buffer.append("&lt;");
                break;
            case '>':
                buffer.append("&gt;");
                break;
            case '&':
                buffer.append("&amp;");
                break;
            default:
                buffer.append(c);
            }
        }

        return buffer.toString();
    }

    private String toJsonp(String name, String message) {
        return "window.parent.app.update({ name: \"" + escape(name) + "\", message: \"" + escape(message) + "\" });\n";
    }    

}
